@@ -40,6 +40,9 @@
  */
 
 #include "config.h"
+#include "openssl/rand.h"
+#include "openssl/hmac.h"
+#include "openssl/sha.h"
 
 //==============================================================================
 // helper functions
@@ -369,7 +372,7 @@ bool _CONFIG::file_write( const char * path )
 				break;
 
 			case DATA_NUMBER:
-				fprintf( fp, "n:%s:%i\n", cfgdat->key.text(), cfgdat->nval );
+				fprintf( fp, "n:%s:%li\n", cfgdat->key.text(), cfgdat->nval );
 				break;
 
 			case DATA_BINARY:
@@ -424,7 +427,7 @@ bool _CONFIG::file_read( const char * path )
 
 		if( fgetc( fp ) != ':' )
 		{
-			printf( "invalid delimiter \'%c\' between type and name ( line %i )\n",
+			printf( "invalid delimiter \'%c\' between type and name ( line %li )\n",
 				next, line );
 			goto parse_fail;
 		}
@@ -456,7 +459,7 @@ bool _CONFIG::file_read( const char * path )
 
 		if( next != ':' )
 		{
-			printf( "invalid delimiter \'%c\' between name and value ( line %i )\n",
+			printf( "invalid delimiter \'%c\' between name and value ( line %li )\n",
 				next, line );
 			goto parse_fail;
 		}
@@ -517,7 +520,7 @@ bool _CONFIG::file_read( const char * path )
 			}
 
 			default:
-				printf( "invalid value type \'%c\' ( line %i )\n",
+				printf( "invalid value type \'%c\' ( line %li )\n",
 					type, line );
 				goto parse_fail;
 		}
@@ -531,7 +534,321 @@ bool _CONFIG::file_read( const char * path )
 
 	parse_fail:
 
-	printf( "parse error in line %i\n", line );
+	printf( "parse error in line %li\n", line );
+
+	fclose( fp );
+
+	return false;
+}
+
+bool read_line_pcf( FILE * fp, BDATA & name, BDATA & data )
+{
+	char	next;
+	BDATA	line;
+
+	name.del();
+	data.del();
+
+	//
+	// read the next line
+	//
+
+	while( true )
+	{
+		next = fgetc( fp );
+
+		if( next == '\r' )
+			continue;
+
+		if( next == '\n' )
+			break;
+
+		if( next == EOF )
+			break;
+
+		line.add( next, 1 );
+	}
+
+	//
+	// check for valid line
+	//
+
+	if( !line.size() )
+	{
+		if( next == EOF )
+			return false;
+		else
+		{
+			name.add( "", 1 );
+			data.add( "", 1 );
+			return true;
+		}
+	}
+
+	//
+	// read the name value
+	//
+
+	while( line.get( &next, 1 ) )
+	{
+		if( !name.size() )
+			if( ( next == ' ' ) || ( next == '!' ) )
+				continue;
+
+		if( next == '=' )
+			break;
+
+		name.add( next, 1 );
+	}
+
+	//
+	// verify the delimiter
+	//
+
+	if( next != '=' )
+		return true;
+
+	//
+	// read the data value
+	//
+
+	while( line.get( &next, 1 ) )
+	{
+		if( !data.size() )
+			if( next == ' ' )
+				continue;
+
+		data.add( next, 1 );
+	}
+
+	//
+	// trim the values
+	//
+
+	if( name.size() )
+		while( name.buff()[ name.size() - 1 ] == ' ' )
+			name.size( name.size() -1 );
+
+	if( data.size() )
+		while( data.buff()[ data.size() - 1 ] == ' ' )
+			data.size( data.size() -1 );
+
+	//
+	// null terminate values
+	//
+
+	name.add( "", 1 );
+	data.add( "", 1 );
+
+	return true;
+}
+
+bool _CONFIG::file_import_pcf( const char * path, bool & need_certs )
+{
+	FILE * fp = fopen( path, "r" );
+	if( fp == NULL )
+		return false;
+
+	//
+	// set some sane defaults
+	//
+
+	set_number( "version", 3 );
+	set_number( "network-ike-port", 500 );
+	set_number( "network-mtu-size", 1380 );
+
+	set_string( "client-auto-mode", "pull", 5 );
+	set_string( "client-iface", "virtual", 8 );
+	set_number( "client-addr-auto", 1 );
+
+	set_string( "network-natt-mode", "enable", 7 );
+	set_number( "network-natt-port", 4500 );
+	set_number( "network-natt-rate", 15 );
+
+	set_string( "network-frag-mode", "disable", 8 );
+	set_number( "network-frag-size", 540 );
+
+	set_number( "network-dpd-enable", 1 );
+	set_number( "network-notify-enable", 1 );
+	set_number( "client-banner-enable", 1 );
+
+	set_string( "auth-method", "mutual-psk-xauth", 17 );
+	set_string( "ident-server-type", "any", 4 );
+
+	set_string( "phase1-exchange", "aggressive", 11 );
+	set_string( "phase1-cipher", "auto", 5 );
+	set_string( "phase1-hash", "auto", 5 );
+	set_number( "phase1-dhgroup", 2 );
+	set_number( "phase1-life-secs", 86400 );
+
+	set_string( "phase2-transform", "auto", 5 );
+	set_string( "phase2-hmac", "auto", 5 );
+	set_number( "phase2-pfsgroup", 0 );
+
+	set_string( "ipcomp-transform", "disabled", 9 );
+
+	set_number( "client-dns-used", 1 );
+	set_number( "client-dns-auto", 1 );
+	set_number( "client-dns-suffix-auto", 1 );
+	set_number( "client-splitdns-used", 1 );
+	set_number( "client-splitdns-auto", 1 );
+	set_number( "client-wins-used", 1 );
+	set_number( "client-wins-auto", 1 );
+
+	set_number( "phase2-life-secs", 3600 );
+	set_number( "phase2-life-kbytes", 0 );
+
+	set_number( "policy-nailed", 0 );
+	set_number( "policy-list-auto", 1 );
+
+	//
+	// parse the file contents
+	//
+
+	long auth_type = 1;
+
+	BDATA	name;
+	BDATA	data;
+
+	while( read_line_pcf( fp, name, data ) )
+	{
+		//
+		// Convert the appropriate values
+		//
+
+		if( !strcasecmp( name.text(), "Host" ) && data.size() )
+			set_string( "network-host", data.text(), data.size() );
+
+		if( !strcasecmp( name.text(), "AuthType" ) && data.size() )
+		{
+			auth_type = atol( data.text() );
+			switch( auth_type )
+			{
+				case 1:
+					set_string( "auth-method", "mutual-psk-xauth", 17 );
+					need_certs = false;
+					break;
+				case 3:
+					set_string( "auth-method", "mutual-rsa-xauth", 17 );
+					need_certs = true;
+					break;
+				case 5:
+					set_string( "auth-method", "hybrid-grp-xauth", 17 );
+					need_certs = true;
+					break;
+				default:
+					goto parse_fail;
+			}
+		}
+
+		if( !strcasecmp( name.text(), "GroupName" ) && data.size() )
+		{
+			set_string( "ident-client-type", "keyid", 6 );
+			set_string( "ident-client-data", data.text(), data.size() );
+		}
+
+		if( !strcasecmp( name.text(), "GroupPwd" ) && data.size() )
+		{
+			data.size( data.size() - 1 );
+			if( !data.hex_decode() )
+				goto parse_fail;
+
+			set_binary( "auth-mutual-psk", data );
+		}
+
+		if( !strcasecmp( name.text(), "enc_GroupPwd" ) && data.size() )
+		{
+			data.size( data.size() - 1 );
+			if( !data.hex_decode() )
+				goto parse_fail;
+
+			//
+			// decrypt cisco password
+			//
+
+			if( data.size() < 48 )
+				goto parse_fail;
+
+			unsigned char key[ 40 ];
+			unsigned char one[ 20 ];
+			unsigned char two[ 20 ];
+			
+			data.get( one, 20 );
+			data.get( two, 20 );
+
+			one[ 19 ] += 1;
+
+			SHA_CTX ctx;
+			SHA1_Init( &ctx );
+			SHA1_Update( &ctx, one, 20 );
+			SHA1_Final( key, &ctx );
+
+			one[ 19 ] += 2;
+
+			SHA1_Init( &ctx );
+			SHA1_Update( &ctx, one, 20 );
+			SHA1_Final( key + 20, &ctx );
+
+			size_t pwlen = data.size() - 40;
+
+			SHA1_Init( &ctx );
+			SHA1_Update( &ctx, data.buff() + 40, pwlen );
+			SHA1_Final( one, &ctx );
+
+			if( memcmp( one, two, 20 ) )
+				goto parse_fail;
+
+			BDATA pwd;
+			data.get( pwd );
+
+			EVP_CIPHER_CTX ctx_cipher;
+			EVP_CIPHER_CTX_init( &ctx_cipher );
+
+			EVP_CipherInit_ex(
+				&ctx_cipher,
+				EVP_des_ede3_cbc(),
+				NULL,
+				key,
+				data.buff(),
+				0 );
+
+			EVP_Cipher(
+				&ctx_cipher,
+				pwd.buff(),
+				pwd.buff(),
+				pwd.size() );
+
+			pwlen -= pwd.buff()[ pwd.size() - 1 ];
+			pwd.size( pwlen );
+
+			set_binary( "auth-mutual-psk", pwd );
+		}
+
+		if( !strcasecmp( name.text(), "DHGroup" ) && data.size() )
+		{
+			long dh_group = atol( data.text() );
+			set_number( "phase1-dhgroup", dh_group );
+		}
+
+		if( !strcasecmp( name.text(), "EnableNat" )  && data.size() )
+		{
+			long enable_nat = atol( data.text() );
+			if( enable_nat )
+				set_string( "network-natt-mode", "enable", 7 );
+			else
+				set_string( "network-natt-mode", "disable", 8 );
+		}
+
+		if( !strcasecmp( name.text(), "Username" ) && data.size() )
+			set_string( "client-saved-username", data.text(), data.size() );
+
+	}
+
+	fclose( fp );
+
+	return true;
+
+	parse_fail:
 
 	fclose( fp );
 
