@@ -313,6 +313,9 @@ void _IKEC::run()
 		return;
 	}
 
+	if( !strcmp( "any", text ) )
+		peer.idtype_r = ISAKMP_ID_NONE;
+
 	if( !strcmp( "asn1dn", text ) )
 		peer.idtype_r = ISAKMP_ID_ASN1_DN;
 
@@ -443,6 +446,12 @@ void _IKEC::run()
 	if( !strcmp( "hybrid-rsa-xauth", text ) )
 		proposal_isakmp.auth_id = HYBRID_AUTH_INIT_RSA;
 
+	if( !strcmp( "hybrid-grp-xauth", text ) )
+	{
+		proposal_isakmp.auth_id = HYBRID_AUTH_INIT_RSA;
+		xconf.opts |= IPSEC_OPTS_CISCO_GRP;
+	}
+
 	if( !strcmp( "mutual-rsa-xauth", text ) )
 		proposal_isakmp.auth_id = XAUTH_AUTH_INIT_RSA;
 
@@ -1083,7 +1092,8 @@ void _IKEC::run()
 	}
 
 	if( ( proposal_isakmp.auth_id == XAUTH_AUTH_INIT_PSK ) ||
-		( proposal_isakmp.auth_id == IKE_AUTH_PRESHARED_KEY ) )
+		( proposal_isakmp.auth_id == IKE_AUTH_PRESHARED_KEY ) ||
+		( xconf.opts & IPSEC_OPTS_CISCO_GRP ) )
 	{
 		// mutual preshared key
 
