@@ -232,10 +232,12 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 			case ISAKMP_PAYLOAD_CERT:
 			{
-				BDATA cert;
-				result = payload_get_cert( packet, ph1->ctype_r, cert );
+				uint8_t	type;
+				BDATA	cert;
+
+				result = payload_get_cert( packet, type, cert );
 				if( result == LIBIKE_OK )
-					ph1->certs_r.add( cert );
+					ph1->certs_r.add( type, cert );
 
 				ph1->xstate |= XSTATE_RECV_CT;
 
@@ -247,15 +249,18 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			//
 
 			case ISAKMP_PAYLOAD_CERT_REQ:
+			{
+				uint8_t	type;
+				BDATA	dn;
 
-				if( ph1->xstate & XSTATE_RECV_CR )
-					log.txt( LLOG_INFO, "<< : ignoring duplicate cert request payload\n" );
-				else
-					result = payload_get_creq( packet, ph1->ctype_l );
+				result = payload_get_creq( packet, type, dn );
+				if( result == LIBIKE_OK )
+					ph1->creqs_r.add( type, dn );
 
 				ph1->xstate |= XSTATE_RECV_CR;
 
 				break;
+			}
 
 			//
 			// signature payload
@@ -1007,12 +1012,56 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							PACKET_IKE packet;
 							packet.write( ph1->cookies, ISAKMP_PAYLOAD_HASH, ph1->exchange, ISAKMP_FLAG_ENCRYPT );
 
+							//
+							// cisco hybrid authentication uses an extra
+							// notification payload containing the group
+							// password hash
+							//
+
+							uint8_t nextp = ph1->natt_pldtype;
+							if( ph1->tunnel->xconf.opts & IPSEC_OPTS_CISCO_GRP )
+								nextp = ISAKMP_PAYLOAD_NOTIFY;
+
 							//
 							// add payloads
 							//
 
 							phase1_gen_hash_i( ph1, ph1->hash_l );
-							payload_add_hash( packet, ph1->hash_l, ph1->natt_pldtype );
+							payload_add_hash( packet, ph1->hash_l, nextp );
+
+							//
+							// optionally add unity notification payload
+							//
+
+							if( ph1->tunnel->xconf.opts & IPSEC_OPTS_CISCO_GRP )
+							{
+								//
+								// generate the group password hash
+								//
+
+								BDATA psk_hash;
+								psk_hash.size( ph1->hash_size );
+
+								HMAC_CTX ctx_prf;
+								HMAC_Init( &ctx_prf, ph1->skeyid.buff(), ( int ) ph1->skeyid.size(), ph1->evp_hash );
+								HMAC_Update( &ctx_prf, ph1->tunnel->peer->psk.buff(), ph1->tunnel->peer->psk.size() );
+								HMAC_Final( &ctx_prf, psk_hash.buff(), NULL );
+								HMAC_cleanup( &ctx_prf );
+
+								//
+								// add the notification payload
+								//
+
+								IKE_NOTIFY notify;
+								notify.type = ISAKMP_PAYLOAD_NOTIFY;
+								notify.code = ISAKMP_N_UNITY_GROUP_HASH;
+								notify.doi = ISAKMP_DOI_IPSEC;
+								notify.proto = ISAKMP_PROTO_ISAKMP;
+								notify.spi.size = sizeof( ph1->cookies );
+								notify.spi.cookies = ph1->cookies;
+								notify.data.set( psk_hash );
+								payload_add_notify( packet, &notify, ph1->natt_pldtype );
+							}
 
 							//
 							// optionally add nat discovery hash payloads
@@ -2052,8 +2101,8 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 	{
 		BDATA vend_unity2;
 		vend_unity2.add( vend_unity );	// base vendor id
-		vend_unity2.add( 0x02, 1 );		// major version
-		vend_unity2.add( 0x04, 1 );		// minor version
+		vend_unity2.add( 0x01, 1 );		// major version
+		vend_unity2.add( 0x00, 1 );		// minor version
 
 		payload_add_vend( packet, vend_unity2, vendpld( vid_count, next ) );
 		log.txt( LLOG_INFO, "ii : local is CISCO UNITY compatible\n" );
@@ -2690,13 +2739,22 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 	ph1->tunnel->lstate |= TSTATE_NATT_FLOAT;
 
 	if( ph1->tunnel->natt_version >= IPSEC_NATT_V02 )
-
 	{
 		//
-		// switch our port to natt
+		// switch our local port to natt
 		//
 
-		ph1->tunnel->saddr_l.saddr4.sin_port = ph1->tunnel->peer->natt_port;
+		if( socket_lookup_port(	ph1->tunnel->saddr_l, true ) != LIBIKE_OK )
+		{
+			log.txt( LLOG_INFO,
+				"ii : unable to locate local nat-t udp port\n" );
+
+			return false;
+		}
+
+		log.txt( LLOG_INFO,
+			"ii : switching to src nat-t udp port %u\n",
+			ntohs( ph1->tunnel->saddr_l.saddr4.sin_port ) );
 
 		//
 		// switch the peer port to natt
@@ -2705,8 +2763,8 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 		ph1->tunnel->saddr_r.saddr4.sin_port = ph1->tunnel->peer->natt_port;
 
 		log.txt( LLOG_INFO,
-			"ii : switching to nat-t udp port %u\n",
-			ntohs( ph1->tunnel->peer->natt_port ) );
+			"ii : switching to dst nat-t udp port %u\n",
+			ntohs( ph1->tunnel->saddr_r.saddr4.sin_port ) );
 
 		//
 		// setup our filter
