@@ -58,23 +58,20 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	// sa for this message id
 	//
 
+	uint32_t msgid = packet.get_msgid();
+
 	IDB_PH2 * ph2 = NULL;
-
-	uint32_t msgid;
-	packet.get_msgid( msgid );
-
-	idb_list_ph2.find(
-		true,
-		&ph2,
-		ph1->tunnel,
-		XCH_STATUS_ANY,
-		XCH_STATUS_ANY,
-		NULL,
-		&msgid,
-		NULL,
-		NULL );
-
-	if( ph2 == NULL )
+	
+	if( !idb_list_ph2.find(
+			true,
+			&ph2,
+			ph1->tunnel,
+			XCH_STATUS_ANY,
+			XCH_STATUS_ANY,
+			NULL,
+			&msgid,
+			NULL,
+			NULL ) )
 	{
 		//
 		// looks like a unique phase 2
@@ -89,7 +86,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		// pahse2 exchange
 		//
 
-		phase2_gen_iv( ph1, ph2->msgid, ph2->iv );
+		ph2->new_msgiv( ph1 );
 
 		//
 		// make sure we respond using the
@@ -101,20 +98,44 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	//
 	// make sure we are not dealing
-	// whith a mature or dead sa
+	// with an sa marked for death
 	//
 
-	if( ( ph1->status() == XCH_STATUS_DEAD ) ||
-	    ( ph2->status() == XCH_STATUS_DEAD ) )
+	if( ph1->status() == XCH_STATUS_DEAD )
 	{
-		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( sa marked for death )\n" );
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( phase1 marked for death )\n" );
 		ph2->dec( true );
 		return LIBIKE_OK;
 	}
 
+	if( ph2->status() == XCH_STATUS_DEAD )
+	{
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( phase2 marked for death )\n" );
+		ph2->dec( true );
+		return LIBIKE_OK;
+	}
+
+	//
+	// make sure we are not dealing
+	// whith an imature phase1 sa
+	//
+
+	if( ph1->status() < XCH_STATUS_MATURE )
+	{
+		log.txt( LLOG_ERROR, "!! : config packet ignored ( phase1 not mature )\n" );
+		ph2->dec( true );
+		return LIBIKE_OK;
+	}
+
+	//
+	// make sure we are not dealing
+	// with a mature sa
+	//
+
 	if( ph2->status() >= XCH_STATUS_MATURE )
 	{
-		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( sa already mature )\n" );
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored, resending last packet ( phase2 already mature )\n" );
+		ph2->resend();
 		ph2->dec( true );
 		return LIBIKE_OK;
 	}
@@ -125,7 +146,8 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	if( packet_ike_decrypt( ph1, packet, &ph2->iv ) != LIBIKE_OK )
 	{
-		log.txt( LLOG_ERROR, "!! : phase2 packet ignored ( packet decryption error )\n" );
+		log.txt( LLOG_ERROR, "!! : phase2 packet ignored, resending last packet ( packet decryption error )\n" );
+		ph2->resend();
 		ph2->dec( true );
 		return LIBIKE_OK;
 	}
@@ -492,7 +514,7 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		//
 
 		if( packet.get_payload_left() )
-			log.txt( LLOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
+			log.txt( LLOG_ERROR, "!! : unprocessed payload data !!!\n" );
 
 		//
 		// check the final paylaod process result
@@ -646,7 +668,6 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 					ph2->status( XCH_STATUS_MATURE, XCH_NORMAL, 0 );
 					ph2->clean();
-					ph2->resend_clear( true );
 				}
 			}
 		}
@@ -670,8 +691,12 @@ long _IKED::process_phase2_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 	// now build and send any response
 	// packets that may be necessary
 	//
+	// NOTE : responder packets are only
+	// sent via the pfkey thread after an
+	// outbound SPI is received
+	//
 
-	if( ( ph2->lstate & LSTATE_HASSPI ) &&
+	if( ph2->initiator &&
 		( ph1->status() != XCH_STATUS_DEAD ) &&
 		( ph2->status() < XCH_STATUS_MATURE ) )
 		process_phase2_send( ph1, ph2 );
@@ -753,7 +778,7 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 			// pahse2 exchange
 			//
 
-			phase2_gen_iv( ph1, ph2->msgid, ph2->iv );
+			ph2->new_msgiv( ph1 );
 
 			//
 			// send packet
@@ -817,7 +842,6 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 			ph2->status( XCH_STATUS_MATURE, XCH_NORMAL, 0 );
 			ph2->clean();
-			ph2->resend_clear( true );
 		}
 	}
 
@@ -954,7 +978,7 @@ long _IKED::process_phase2_send( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 			// send packet
 			//
 
-			packet_ike_send( ph1, ph2, packet, true );
+			packet_ike_send( ph1, ph2, packet, false );
 
 			//
 			// update sa state
@@ -1502,6 +1526,34 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 
 	if( ph2->dhgr_id )
 	{
+		if( level >= LLOG_DECODE )
+		{
+			BDATA prv;
+			prv.size( ph2->dh_size );
+			BN_bn2bin( ph2->dh->priv_key, prv.buff() );
+
+			log.bin(
+				LLOG_DECODE,
+				LLOG_DECODE,
+				prv.buff(),
+				prv.size(),
+				"ii : computed PFS DH private key" );
+
+			log.bin(
+				LLOG_DECODE,
+				LLOG_DECODE,
+				ph2->xl.buff(),
+				ph2->xl.size(),
+				"ii : computed PFS DH public key" );
+
+			log.bin(
+				LLOG_DECODE,
+				LLOG_DECODE,
+				ph2->xr.buff(),
+				ph2->xr.size(),
+				"ii : received PFS DH public key" );
+		}
+
 		//
 		// validate the dh group size
 		//
@@ -1509,26 +1561,50 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 )
 		if( ph2->xr.size() != ph2->dh_size )
 		{
 			log.txt( LLOG_ERROR,
-				"!! : dh group size mismatch ( %d != %d )\n",
+				"!! : PFS DH group size mismatch ( %d != %d )\n",
 				ph2->xr.size(),
 				ph2->dh_size );
 
+			ph2->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_FORMAT, 0 );
 			return LIBIKE_FAILED;
 		}
 
+		//
+		// determine shared secret
+		//
+
 		BIGNUM * gx = BN_new();
 		BN_bin2bn( ph2->xr.buff(), ph2->dh_size, gx );
 
 		shared.size( ph2->dh_size );
-		DH_compute_key( shared.buff(), gx, ph2->dh );
+		long result = DH_compute_key( shared.buff(), gx, ph2->dh );
 		BN_free( gx );
 
+		if( result < 0 )
+		{
+			log.txt( LLOG_ERROR,
+				"!! : failed to compute PFS DH shared secret\n" );
+
+			ph2->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_CRYPTO, 0 );
+			return LIBIKE_FAILED;
+		}
+
+		//
+		// fixup shared secret buffer alignment
+		//
+
+		if( ph2->dh_size > result )
+		{
+			shared.size( result );
+			shared.ins( 0, ph2->dh_size - result );
+		}
+
 		log.bin(
 			LLOG_DEBUG,
 			LLOG_DECODE,
 			shared.buff(),
 			shared.size(),
-			"== : pfs dh shared secret" );
+			"== : PFS DH shared secret" );
 	}
 
 	//
@@ -1788,34 +1864,3 @@ long _IKED::phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSA
 
 	return LIBIKE_OK;
 }
-
-long _IKED::phase2_gen_iv( IDB_PH1 * ph1, unsigned long msgid, BDATA & iv )
-{
-	//
-	// create new informational iv
-	//
-
-	if( ph1->evp_cipher != NULL )
-	{
-		unsigned char iv_data[ EVP_MAX_MD_SIZE ];
-		unsigned long iv_size = EVP_CIPHER_iv_length( ph1->evp_cipher );
-
-		EVP_MD_CTX ctx_hash;
-		EVP_DigestInit( &ctx_hash, ph1->evp_hash );
-		EVP_DigestUpdate( &ctx_hash, ph1->iv.buff(), ph1->iv.size() );
-		EVP_DigestUpdate( &ctx_hash, &msgid, 4 );
-		EVP_DigestFinal( &ctx_hash, iv_data, NULL );
-		EVP_MD_CTX_cleanup( &ctx_hash );
-
-		iv.set( iv_data, iv_size );
-
-		log.bin(
-			LLOG_DEBUG,
-			LLOG_DECODE,
-			iv.buff(),
-			iv.size(),
-			"== : new phase2 iv" );
-	}
-
-	return LIBIKE_OK;
-}
