@@ -329,6 +329,7 @@
 // cisco high availability
 
 #define ISAKMP_N_UNITY_LOAD_BALANCE	40501
+#define ISAKMP_N_UNITY_GROUP_HASH	40503
 
 // dead peer detection
 
@@ -486,7 +487,9 @@
 #define IPSEC_OPTS_DOMAIN		0x0080
 #define IPSEC_OPTS_SPLITDNS		0x0100
 #define IPSEC_OPTS_SAVEPW		0x0200
-#define IPSEC_OPTS_VEND_CHKPT	0x0400
+#define IPSEC_OPTS_CISCO_UDP	0x0400
+#define IPSEC_OPTS_CISCO_GRP	0x0800
+#define IPSEC_OPTS_VEND_CHKPT	0x1000
 
 #define IPSEC_DNSS_MAX			4
 #define IPSEC_NBNS_MAX			2
@@ -614,8 +617,10 @@ typedef struct _IKE_PEER
 
 typedef struct _IKE_XAUTH
 {
-	BDATA	user;
-	BDATA	pass;
+	uint16_t	type;
+	BDATA		user;
+	BDATA		pass;
+	BDATA		hash;
 
 	BDATA	context;
 
