@@ -244,7 +244,6 @@ void _IDB_LIST_PH2::flush()
 _IDB_PH2::_IDB_PH2( IDB_TUNNEL * set_tunnel, bool set_initiator, uint32_t set_msgid, uint32_t set_seqid_in )
 {
 	memset( &cookies, 0, sizeof( cookies ) );
-	msgid = 0;
 	seqid_in = 0;
 	seqid_out = 0;
 	nailed_plcyid = 0;
@@ -272,7 +271,7 @@ _IDB_PH2::_IDB_PH2( IDB_TUNNEL * set_tunnel, bool set_initiator, uint32_t set_ms
 	if( set_msgid )
 		msgid = set_msgid;
 	else
-		iked.rand_bytes( &msgid, sizeof( msgid ) );
+		new_msgid();
 
 	//
 	// initialize seqids
@@ -323,9 +322,9 @@ _IDB_PH2::~_IDB_PH2()
 // abstract functions from parent class
 //
 
-char * _IDB_PH2::name()
+const char * _IDB_PH2::name()
 {
-	static char * xname = "phase2";
+	static const char * xname = "phase2";
 	return xname;
 }
 
@@ -344,7 +343,7 @@ void _IDB_PH2::end()
 	// clear the resend queue
 	//
 
-	resend_clear( false );
+	resend_clear( false, true );
 
 	//
 	// remove scheduled events
@@ -429,12 +428,25 @@ bool _IDB_PH2::setup_dhgrp()
 
 	if( dhgr_id )
 	{
-		dh = dh_setup_group( dhgr_id );
-		dh_size = BN_num_bytes( dh->p );
-		dh_create_e( dh, dh_size );
+		if( !dh_init( dhgr_id, &dh, &dh_size ) )
+		{
+			iked.log.txt( LLOG_ERROR, "ii : failed to setup PFS DH group\n" );
+			return false;
+		}
 
 		xl.size( dh_size );
-		BN_bn2bin( dh->pub_key, xl.buff() );
+		long result = BN_bn2bin( dh->pub_key, xl.buff() );
+
+		//
+		// fixup public buffer alignment
+		//
+
+		if( dh_size > result )
+		{
+			iked.log.txt( LLOG_DEBUG, "ww : short PFS DH public value\n" );
+			xl.size( result );
+			xl.ins( 0, dh_size - result );
+		}
 	}
 
 	return true;
