@@ -85,10 +85,9 @@ void _ITH_EVENT_TUNDPD::next()
 	if( iked.ith_timer.del( this ) )
 	{
 		//
-		// move to the next sequence
+		// reset the attempt counter
 		//
 
-		sequence++;
 		attempt = 0;
 
 		//
@@ -176,10 +175,10 @@ bool _ITH_EVENT_TUNDPD::func()
 	}
 
 	//
-	// add sequence number and send
+	// add new sequence number and send
 	//
 
-	uint32_t dpdseq = htonl( sequence );
+	uint32_t dpdseq = htonl( ++sequence );
 
 	BDATA bdata;
 	bdata.add( &dpdseq, sizeof( dpdseq ) );
@@ -280,6 +279,8 @@ bool _ITH_EVENT_TUNSTATS::func()
 
 	if( !( tunnel->close ) && ( tunnel->tstate & TSTATE_VNET_ENABLE ) )
 	{
+		tunnel->stats.peer = tunnel->saddr_r;
+
 		IKEI_MSG msg;
 		msg.set_stats( &tunnel->stats );
 		tunnel->ikei->send_message( msg );
@@ -461,9 +462,9 @@ _IDB_TUNNEL::~_IDB_TUNNEL()
 // abstract functions from parent class
 //
 
-char * _IDB_TUNNEL::name()
+const char * _IDB_TUNNEL::name()
 {
-	static char * xname = "tunnel";
+	static const char * xname = "tunnel";
 	return xname;
 }
 
