@@ -156,6 +156,20 @@ long _IKED::loop_ike_nwork()
 			unsigned short port_src = htons( saddr_src.saddr4.sin_port );
 			unsigned short port_dst = htons( saddr_dst.saddr4.sin_port );
 
+			//
+			// check for NAT-T keep alive
+			//
+
+			if( packet_udp.size() < sizeof( IKE_HEADER ) )
+			{
+				log.txt( LLOG_DEBUG,
+					"<- : recv NAT-T:KEEP-ALIVE packet %s:%u -> %s:%u\n",
+					txtaddr_src, port_src,
+					txtaddr_dst, port_dst );
+
+				continue;
+			}
+
 			//
 			// examine the packet contents
 			// for a NAT-T non-ESP marker
@@ -207,20 +221,6 @@ long _IKED::loop_ike_nwork()
 				PACKET_IKE packet_ike;
 				packet_udp.get( packet_ike );
 
-				//
-				// check for NAT-T keep alive
-				//
-
-				if( packet_ike.size() < sizeof( IKE_HEADER ) )
-				{
-					log.txt( LLOG_DEBUG,
-						"<- : recv NAT-T:KEEP-ALIVE packet %s:%u -> %s:%u\n",
-						txtaddr_src, port_src,
-						txtaddr_dst, port_dst );
-
-					continue;
-				}
-
 				//
 				// process the ike packet
 				//
@@ -254,19 +254,22 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 	// read packet header
 	//
 
-	log.txt( LLOG_DEBUG,
-		"ii : parsing ike packet header\n" );
-
 	IKE_COOKIES	cookies;
 	uint8_t		payload;
 	uint8_t		exchange;
 	uint8_t		flags;
 
-	packet.read(
-		cookies,
-		payload,
-		exchange,
-		flags );
+	if( !packet.read(
+			cookies,
+			payload,
+			exchange,
+			flags ) )
+	{
+		log.txt( LLOG_ERROR,
+			"!! : invalid ISAKMP header\n" );
+
+		return LIBIKE_OK;
+	}
 
 	char txtaddr_src[ LIBIKE_MAX_TEXTADDR ];
 	text_addr( txtaddr_src, &saddr_src, false );
@@ -276,9 +279,6 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 	// sa for this packet
 	//
 
-	log.txt( LLOG_DEBUG,
-		"ii : attempting to locate phase1 sa for packet\n" );
-
 	IDB_PH1 * ph1 = NULL;
 
 	if( !idb_list_ph1.find(
@@ -309,8 +309,8 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 		if( !null_cookie )
 		{
 			log.txt( LLOG_INFO,
-				"XX : ike packet from %s ignored, unknown phase1 sa for peer\n"
-				"XX : %08x%08x:%08x%08x\n",
+				"ww : ike packet from %s ignored, unknown phase1 sa for peer\n"
+				"ww : %08x%08x:%08x%08x\n",
 				txtaddr_src,
 				htonl( *( long * ) &cookies.i[ 0 ] ),
 				htonl( *( long * ) &cookies.i[ 4 ] ),
@@ -351,7 +351,7 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 					&saddr_src ) )
 			{
 				log.txt( LLOG_INFO,
-					"XX : ike packet from %s ignored, no matching definition for peer\n",
+					"ww : ike packet from %s ignored, no matching definition for peer\n",
 					txtaddr_src );
 
 				return LIBIKE_OK;
@@ -362,7 +362,7 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 			if( tunnel == NULL )
 			{
 				log.txt( LLOG_INFO,
-					"XX : ike packet from %s ignored, unable to create tunnel object\n",
+					"ww : ike packet from %s ignored, unable to create tunnel object\n",
 					txtaddr_src );
 
 				peer->dec( true );
@@ -381,7 +381,7 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 		if( exchange != tunnel->peer->exchange )
 		{
 			log.txt( LLOG_INFO,
-				"XX : ike packet from %s ignored, exchange type mismatch for peer\n",
+				"ww : ike packet from %s ignored, exchange type mismatch for peer\n",
 				txtaddr_src );
 
 			tunnel->dec( true );
@@ -392,20 +392,17 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 			( tunnel->peer->contact != IPSEC_CONTACT_BOTH ) )
 		{
 			log.txt( LLOG_INFO,
-				"XX : ike packet from %s ignored, contact is denied for peer\n",
+				"ww : ike packet from %s ignored, contact is denied for peer\n",
 				txtaddr_src );
 
 			tunnel->dec( true );
 			return LIBIKE_OK;
 		}
 
-		uint32_t msgid;
-		packet.get_msgid( msgid );
-
-		if( msgid )
+		if( packet.get_msgid() )
 		{
 			log.txt( LLOG_INFO,
-				"XX : ike packet from %s ignored, invalid message id for exchange type\n",
+				"ww : ike packet from %s ignored, invalid message id for exchange type\n",
 				txtaddr_src );
 
 			tunnel->dec( true );
@@ -425,7 +422,7 @@ long _IKED::process_ike_recv( PACKET_IKE & packet, IKE_SADDR & saddr_src, IKE_SA
 		if( ph1 == NULL )
 		{
 			log.txt( LLOG_INFO,
-				"XX : ike packet from %s ignored, unable to create phase1 handle\n",
+				"ww : ike packet from %s ignored, unable to create phase1 handle\n",
 				txtaddr_src );
 
 			tunnel->dec( true );
