@@ -741,8 +741,9 @@ const char * _IKED::find_name( long type, long id )
 			static const char * notify32 = "REPLAY-STATUS";
 			static const char * notify33 = "INITIAL-CONTACT";
 			static const char * notify34 = "UNITY-LOAD-BALANCE";
-			static const char * notify35 = "DPDV1-R-U-THERE";
-			static const char * notify36 = "DPDV1-R-U-THERE-ACK";
+			static const char * notify35 = "UNITY-GROUP-HASH";
+			static const char * notify36 = "DPDV1-R-U-THERE";
+			static const char * notify37 = "DPDV1-R-U-THERE-ACK";
 			
 			switch( id )
 			{
@@ -848,12 +849,15 @@ const char * _IKED::find_name( long type, long id )
 				case ISAKMP_N_UNITY_LOAD_BALANCE:
 					return notify34;
 
-				case ISAKMP_N_DPD_R_U_THERE:
+				case ISAKMP_N_UNITY_GROUP_HASH:
 					return notify35;
 
-				case ISAKMP_N_DPD_R_U_THERE_ACK:
+				case ISAKMP_N_DPD_R_U_THERE:
 					return notify36;
 
+				case ISAKMP_N_DPD_R_U_THERE_ACK:
+					return notify37;
+
 				default:
 					return unknown2;
 			}
