@@ -79,7 +79,6 @@
 #  include <sys/linker.h>
 # endif
 # include "compat/winstring.h"
-# include "conf.parse.hpp"
 # ifndef SOCKET
 #  define SOCKET int
 # endif
@@ -140,18 +139,11 @@
 #define PATH_DEBUG		"/var/log"
 #define MAX_PATH		1024
 
-// Conf parser definition
-
-#define YY_DECL										\
-	yy::conf_parser::token_type						\
-	yylex( yy::conf_parser::semantic_type * yylval,	\
-	yy::conf_parser::location_type * yylloc,		\
-	IKED & iked )
-
-YY_DECL;
+namespace yy{ class conf_parser; };
 
 #endif
 
+
 //
 // IKED constants
 //
@@ -237,51 +229,52 @@ YY_DECL;
 #define NAME_IDENT				16
 #define NAME_NOTIFY				17
 
-#define XSTATE_SENT_SA			0x000000001
-#define XSTATE_SENT_KE			0x000000002
-#define XSTATE_SENT_NO			0x000000004
-#define XSTATE_SENT_ID			0x000000008
-#define XSTATE_SENT_CT			0x000000010
-#define XSTATE_SENT_CR			0x000000020
-#define XSTATE_SENT_SI			0x000000040
-#define XSTATE_SENT_HA			0x000000080
-#define XSTATE_RECV_SA			0x000000100
-#define XSTATE_RECV_KE			0x000000200
-#define XSTATE_RECV_NO			0x000000400
-#define XSTATE_RECV_ID			0x000000800
-#define XSTATE_RECV_SI			0x000001000
-#define XSTATE_RECV_CT			0x000002000
-#define XSTATE_RECV_CR			0x000004000
-#define XSTATE_RECV_ND			0x000008000
-#define XSTATE_RECV_IDL			0x000010000
-#define XSTATE_RECV_IDR			0x000020000
-#define XSTATE_RECV_HA			0x000040000
-#define XSTATE_RECV_LP			0x000080000
-#define XSTATE_SENT_LP			0x000100000
+#define XSTATE_SENT_SA			0x00000001
+#define XSTATE_SENT_KE			0x00000002
+#define XSTATE_SENT_NO			0x00000004
+#define XSTATE_SENT_ID			0x00000008
+#define XSTATE_SENT_CT			0x00000010
+#define XSTATE_SENT_CR			0x00000020
+#define XSTATE_SENT_SI			0x00000040
+#define XSTATE_SENT_HA			0x00000080
+#define XSTATE_RECV_SA			0x00000100
+#define XSTATE_RECV_KE			0x00000200
+#define XSTATE_RECV_NO			0x00000400
+#define XSTATE_RECV_ID			0x00000800
+#define XSTATE_RECV_SI			0x00001000
+#define XSTATE_RECV_CT			0x00002000
+#define XSTATE_RECV_CR			0x00004000
+#define XSTATE_RECV_ND			0x00008000
+#define XSTATE_RECV_IDL			0x00010000
+#define XSTATE_RECV_IDR			0x00020000
+#define XSTATE_RECV_HA			0x00040000
+#define XSTATE_RECV_LP			0x00080000
+#define XSTATE_SENT_LP			0x00100000
 
-#define CSTATE_RECV_XUSER		0x000000001
-#define CSTATE_SENT_XUSER		0x000000002
-#define CSTATE_RECV_XPASS		0x000000004
-#define CSTATE_SENT_XPASS		0x000000008
-#define CSTATE_RECV_XRSLT		0x000000010
-#define CSTATE_SENT_XRSLT		0x000000020
-#define CSTATE_RECV_XCONF		0x000000040
-#define CSTATE_SENT_XCONF		0x000000080
-#define CSTATE_RECV_ACK			0x000000100
-#define CSTATE_SENT_ACK			0x000000200
+#define CSTATE_RECV_XUSER		0x00000001
+#define CSTATE_SENT_XUSER		0x00000002
+#define CSTATE_RECV_XPASS		0x00000004
+#define CSTATE_SENT_XPASS		0x00000008
+#define CSTATE_RECV_XRSLT		0x00000010
+#define CSTATE_SENT_XRSLT		0x00000020
+#define CSTATE_RECV_XCONF		0x00000040
+#define CSTATE_SENT_XCONF		0x00000080
+#define CSTATE_RECV_ACK			0x00000100
+#define CSTATE_SENT_ACK			0x00000200
+#define CSTATE_USE_PASSCODE		0x80000000
 
-#define LSTATE_CHKPROP			0x000000001		// proposal verified
-#define LSTATE_CHKHASH			0x000000002		// hash verified
-#define LSTATE_CHKIDS			0x000000004		// identity verified
-#define LSTATE_GENNATD			0x000000008		// natt discovery generated
-#define LSTATE_HASKEYS			0x000000010		// keys generated
-#define LSTATE_CLAIMLT			0x000000020		// claim reponder lifetime
+#define LSTATE_CHKPROP			0x00000001		// proposal verified
+#define LSTATE_CHKHASH			0x00000002		// hash verified
+#define LSTATE_CHKIDS			0x00000004		// identity verified
+#define LSTATE_GENNATD			0x00000008		// natt discovery generated
+#define LSTATE_HASKEYS			0x00000010		// keys generated
+#define LSTATE_CLAIMLT			0x00000020		// claim reponder lifetime
 
-#define TSTATE_NATT_FLOAT		0x000000001
-#define TSTATE_INITIALIZED		0x000000002
-#define TSTATE_VNET_CONFIG		0x000000004
-#define TSTATE_VNET_ENABLE		0x000000008
-#define TSTATE_POLICY_INIT		0x000000010
+#define TSTATE_NATT_FLOAT		0x00000001
+#define TSTATE_INITIALIZED		0x00000002
+#define TSTATE_VNET_CONFIG		0x00000004
+#define TSTATE_VNET_ENABLE		0x00000008
+#define TSTATE_POLICY_INIT		0x00000010
 
 #define DSTATE_ACTIVE			0
 #define DSTATE_TERMINATE		1
@@ -303,6 +296,7 @@ typedef struct _SOCK_INFO : public IDB_ENTRY
 {
 	int			sock;
 	IKE_SADDR	saddr;
+	bool		natt;
 
 }SOCK_INFO;
 
@@ -496,9 +490,10 @@ typedef class _IKED
 
 	long	socket_init();
 	void	socket_done();
-	long	socket_create( IKE_SADDR & saddr, bool encap );
-	long	socket_locate( IKE_SADDR & saddr );
+	long	socket_create( IKE_SADDR & saddr, bool natt );
 	long	socket_select( unsigned long timeout );
+	long	socket_lookup_addr( IKE_SADDR & saddr_l, IKE_SADDR & saddr_r );
+	long	socket_lookup_port( IKE_SADDR & saddr_l, bool natt );
 
 #ifdef WIN32
 
@@ -536,9 +531,6 @@ typedef class _IKED
 	void	text_ph1id( char * text, IKE_PH1ID * ph1id );
 	void	text_ph2id( char * text, IKE_PH2ID * ph2id );
 
-	bool	find_addr_r( sockaddr_in & raddr, unsigned short rport, char * rname );
-	bool	find_addr_l( IKE_SADDR & saddr_r, IKE_SADDR & addr_l, unsigned short lport );
-
 	// config file loader
 
 	bool	conf_load( const char * path, bool trace = false );
@@ -725,7 +717,7 @@ typedef class _IKED
 	long	payload_get_cert( PACKET_IKE & packet, uint8_t & type, BDATA & cert );
 
 	long	payload_add_creq( PACKET_IKE & packet, uint8_t type, uint8_t next );
-	long	payload_get_creq( PACKET_IKE & packet, uint8_t & type );
+	long	payload_get_creq( PACKET_IKE & packet, uint8_t & type, BDATA & dn );
 
 	long	payload_add_sign( PACKET_IKE & packet, BDATA & sign, uint8_t next );
 	long	payload_get_sign( PACKET_IKE & packet, BDATA & sign );
