@@ -65,8 +65,21 @@ bool _LOG::write_buff( char * buff, size_t size )
 		struct tm *	ltime;
 
 		time( &ctime );
+
+#ifdef WIN32
+
+		struct tm ltm;
+		ltime = &ltm;
+		localtime_s( ltime, &ctime );
+
+#endif
+
+#ifdef UNIX
+
 		ltime = localtime( &ctime );
 
+#endif
+
 		tlen = strftime( tbuff, LOG_MAX_TXT, "%y/%m/%d %H:%M:%S ", ltime );
 	}
 
@@ -77,7 +90,7 @@ bool _LOG::write_buff( char * buff, size_t size )
 	//
 
 	if( log_flags & LOGFLAG_ECHO )
-		printf( buff );
+		printf( "%s", buff );
 
 	//
 	// log individual lines
@@ -134,7 +147,7 @@ bool _LOG::write_line( char * buff, size_t size )
 #ifdef UNIX
 
 	if( log_flags & LOGFLAG_SYSTEM )
-		syslog( LOG_NOTICE, buff );
+		syslog( LOG_NOTICE, "%s", buff );
 	else
 	{
 		if( fp != NULL )
